# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/19 
# @Function:
import numpy as np

from backend import Backend


class BackendTensorRT(Backend):
    def __init__(self, *args, **kwargs):
        self.model_dir = kwargs.get("model_dir")
        self.device_id = kwargs.get("device_id")
        self.model = None
        self.backend = "tensorrt"
        super().__init__(*args)

    def build(self, **kwargs):
        from_platform = kwargs.get("from_platform")
        module = kwargs.get("module")
        dynamic_axes = kwargs.get("dynamic_axes")
        opset_version = kwargs.get("opset_version")
        example_inputs = kwargs.get("example_inputs")
        shape = kwargs.get("shape")

        self.pop(kwargs, "from_platform")
        self.pop(kwargs, "module")
        self.pop(kwargs, "dynamic_axes")
        self.pop(kwargs, "opset_version")
        self.pop(kwargs, "example_inputs")
        self.pop(kwargs, "shape")

        self.pop(kwargs, "model_name")
        self.pop(kwargs, "model_version")
        self.pop(kwargs, "http_inputs")
        self.pop(kwargs, "http_outputs")
        self.pop(kwargs, "concurrency")

        target_files = self.get_file_by_suffix(model_dir=self.model_dir, suffix=".trt")
        if len(target_files) == 0:
            print("tensorrt文件不存在，正在导出trt...")
            target_files = self.get_file_by_suffix(model_dir=self.model_dir, suffix=".onnx")
            if len(target_files) == 0:
                from backend_onnx import BackendONNX
                self.backend = BackendONNX(model_dir=self.model_dir, device_id=self.device_id)
                self.backend.build(from_platform=from_platform, module=module, dynamic_axes=dynamic_axes,
                                          opset_version=opset_version, example_inputs=example_inputs, **kwargs)
            assert shape is not None, "导出trt时，shape不能为空."
            from zhousflib.ann.onnx import onnx_to_trt
            onnx_file_path = self.model_dir.joinpath("model.onnx")
            save_trt_path = self.model_dir.joinpath("model.trt")
            onnx_to_trt.convert_trt(onnx_file_path=onnx_file_path, save_trt_path=save_trt_path, shape=shape)
            print("导出tensorrt文件成功：{0}".format(save_trt_path))
            target_files.append(save_trt_path)
        else:
            print("tensorrt文件存在：{0}.".format(target_files[0]))

        if kwargs.get("autoload", True):
            from zhousflib.ann.tensorrt import tensorrt_infer
            self.model = tensorrt_infer.RTInfer(trt_file_path=self.model_dir.joinpath("model.trt"),
                                                device_id=self.device_id, use_stack=True)
            print("加载tensorrt成功：{0}.".format(target_files[0]))
        return target_files[0]

    def inference(self, inputs_list: list):
        inputs = []
        for i, input_ in enumerate(inputs_list):
            inputs.append(self.to_numpy(inputs_list[i].int()))
        result = self.model.infer(input_arr=np.asarray(inputs))
        return result[0]



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/19 
# @Function:
import numpy as np

from backend import Backend


class BackendHttp(Backend):
    def __init__(self, *args, **kwargs):
        self.model_dir = kwargs.get("model_dir")
        self.device_id = kwargs.get("device_id")
        self.model = None
        self.model_name = None
        self.model_version = None
        self.http_inputs = None
        self.http_outputs = None
        self.backend = "http"
        super().__init__(*args)

    def build(self, url: str, model_name: str, model_version: str, http_inputs: list, http_outputs: list, **kwargs):
        self.model_name = model_name
        self.model_version = model_version
        self.http_inputs = http_inputs
        self.http_outputs = http_outputs

        self.pop(kwargs, "from_platform")
        self.pop(kwargs, "module")
        self.pop(kwargs, "dynamic_axes")
        self.pop(kwargs, "opset_version")
        self.pop(kwargs, "example_inputs")
        self.pop(kwargs, "shape")

        self.pop(kwargs, "model_name")
        self.pop(kwargs, "model_version")
        self.pop(kwargs, "http_inputs")
        self.pop(kwargs, "http_outputs")
        self.pop(kwargs, "concurrency")

        assert url is not None, "设置http后端时，url不能为空."
        assert self.model_name is not None, "设置http后端时，model_name不能为空."
        assert self.model_version is not None, "设置http后端时，model_version不能为空."

        concurrency = kwargs.get("concurrency", 1)
        from zhousflib.infer_framework.triton import client_http
        self.model = client_http.ClientHttp(url=url, concurrency=concurrency, **kwargs)

    def inference(self, inputs_list: list):
        inputs = []
        for i, input_ in enumerate(inputs_list):
            inputs.append(self.to_numpy(inputs_list[i].int()))
        # INT8|INT16|INT32|INT64|UINT8|UINT16|UINT32|UINT64|FP16|FP32|FP64
        d_type = self.http_inputs[0][1]
        if d_type == "INT16":
            data_arr = np.asarray(inputs, dtype=np.int16)
        elif d_type == "INT32":
            data_arr = np.asarray(inputs, dtype=np.int32)
        elif d_type == "INT64":
            data_arr = np.asarray(inputs, dtype=np.int64)
        elif d_type == "FP16":
            data_arr = np.asarray(inputs, dtype=np.float16)
        elif d_type == "FP32":
            data_arr = np.asarray(inputs, dtype=np.float32)
        elif d_type == "FP64":
            data_arr = np.asarray(inputs, dtype=np.float64)
        else:
            assert "暂不支持：{0}".format(d_type)
            data_arr = np.asarray(inputs, dtype=np.int64)
        input_list_ = []
        for i in range(0, len(data_arr)):
            input_list_.append(self.model.build_input(name=self.http_inputs[i][0], data=data_arr[i], datatype=self.http_inputs[i][1]))
        output_list_ = []
        for i in range(0, len(self.http_outputs)):
            output_list_.append(self.model.build_output(name=self.http_outputs[i]))
        """
        同步
        """
        result = self.model.infer_sync(model_name=self.model_name, model_version=self.model_version, inputs=input_list_, outputs=output_list_)
        if len(self.http_outputs) == 1:
            return result.as_numpy(self.http_outputs[0])
        else:
            results = []
            for i in range(0, len(self.http_outputs)):
                results.append(result.as_numpy(self.http_outputs[i]))
            return np.asarray(results)


# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import json
from prettytable import PrettyTable


def data_statistics(labelme_dirs: list):
    """
    数据分布统计
    :param labelme_dirs: 数据集目录 Path
    :return:
    """
    label_list = {}
    for labelme_dir in labelme_dirs:
        for json_file in labelme_dir.rglob("*.json"):
            print(json_file)
            with json_file.open("r", encoding="utf-8") as f:
                data = json.load(f)
                for shape in data["shapes"]:
                    label = shape["label"]
                    if len(label) and label:
                        if label not in label_list:
                            label_list[label] = 1
                        else:
                            label_list[label] += 1

    table = PrettyTable(field_names=["label", "count"])
    for key in label_list:
        table.add_row([key, label_list.get(key)])
    print(table)



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/21 
# @Function:
import torch

from zhousflib.ann import check_device_id


"""
############## 【安装torch】 ##############
选择版本：https://pytorch.org/get-started/locally/
【cpu】
pip install torch==1.13.1+cpu torchvision==0.14.1+cpu torchaudio==0.13.1 --extra-index-url https://download.pytorch.org/whl/cpu
【gpu】
pip install torch==1.13.1+cu117 torchvision==0.14.1+cu117 torchaudio==0.13.1 --extra-index-url https://download.pytorch.org/whl/cu117
验证：
import torch
print(torch.__version__)

"""


def check_cuda():
    assert torch.cuda.is_available(), 'torch.cuda is not available, please check device_id.'


def get_device(device_id: int = -1):
    """
    :param device_id: cpu上运行：-1 | gpu上运行：0 or 1 or 2...
    :return:
    """
    check_device_id(device_id)
    if device_id == -1:
        map_location = torch.device('cpu')
    else:
        check_cuda()
        map_location = torch.device("cuda:{0}".format(device_id))
    """
    Map tensors from GPU 1 to GPU 0
    map_location={'cuda:1': 'cuda:0'}
    https://pytorch.org/docs/stable/generated/torch.load.html
    """
    return map_location



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/21 
# @Function:
from pathlib import Path

from transformers import AutoModel, AutoConfig, AutoTokenizer, BertTokenizerFast, BertTokenizer

"""
############## 【安装transformers(from HuggingFace)】 ##############
# 注意版本要一致，不然会报错：Unexpected key(s) in state_dict: "bert.embeddings.position_ids".
pip install transformers==4.30.2 -i http://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com
"""


def load_config(model_dir: Path):
    """
    加载配置
    :param model_dir:
    :return:
    """
    return AutoConfig.from_pretrained(pretrained_model_name_or_path=model_dir)


def load_model(model_dir: Path, config):
    """
    加载模型
    :param model_dir:
    :param config:
    :return:
    """
    return AutoModel.from_pretrained(model_dir, config=config)


def load_tokenizer(model_dir: Path):
    """
    加载tokenizer
    :param model_dir: 模型目录
    :return:
    """
    tokenizer = None
    try:
        # 加载tokenizer
        tokenizer = AutoTokenizer.from_pretrained(model_dir)
    except Exception as e:
        pass
    return tokenizer


def load_bert_tokenizer(model_dir: Path, use_fast: bool = True):
    """
    加载BertTokenizer
    :param model_dir: 模型目录
    :param use_fast:
    :return:
    """
    tokenizer = None
    try:
        # 加载tokenizer
        if use_fast:
            tokenizer = BertTokenizerFast.from_pretrained(model_dir)
        else:
            tokenizer = BertTokenizer.from_pretrained(model_dir)
    except Exception as e:
        pass
    return tokenizer



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/21 
# @Function:
"""
【onnx && cuda的版本对应关系】
onnx对应cuda的版本：https://onnxruntime.ai/docs/execution-providers/CUDA-ExecutionProvider.html#requirements
注意onnxruntime与opset版本的对应关系


############## 【安装onnxruntime】 ##############
选择版本：https://onnxruntime.ai/docs/execution-providers/CUDA-ExecutionProvider.html#requirements
【cpu】
pip install onnxruntime==1.13.1 -i http://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com
【gpu】
pip install onnxruntime-gpu==1.13.1 -i http://mirrors.aliyun.com/pypi/simple/ --trusted-host mirrors.aliyun.com
验证：
import onnxruntime
onnxruntime.get_device()


############## 【验证导出onnx是否正确】 ##############
可视化网络结构：https://netron.app/
当output有if条件则会存在问题，更换opset版本(opset=10)或降低torch版本(1.8.0)
"""



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/11/27 
# @Function:
# 注意：导出和加载时torch版本要一致，不然会报错：file not found: traced_model_cpu/version
import shutil
from pathlib import Path

import torch
from transformers import AutoModel, AutoConfig, AutoTokenizer

from zhousflib.ann.torch import check_cuda, check_device_id, get_device


def load_script_model(model_dir: Path, device_id: int = -1):
    """
    加载模型
    :param model_dir: 模型目录
    :param device_id: cpu上运行：-1 | gpu上运行：0 or 1 or 2...
    :return:
    """
    device = get_device(device_id)
    pt_file = None
    bin_file = None
    model = None
    tokenizer = None
    state_dict = None
    for file in model_dir.glob("*.pt"):
        pt_file = file
        break
    for file in model_dir.glob("*.bin"):
        bin_file = file
    if pt_file:
        # 加载模型
        model = torch.jit.load(pt_file, map_location=device)
        model.eval()
    if bin_file:
        # 加载模型权重
        state_dict = torch.load(bin_file, map_location=device)
    try:
        # 加载tokenizer
        tokenizer = AutoTokenizer.from_pretrained(model_dir)
    except Exception as e:
        pass
    return model, tokenizer, state_dict


def convert_script_model(model_dir: Path, export_dir: Path, device_id: int = -1, example_inputs=None,
                         module: torch.nn.Module = None, **kwargs):
    """
    转torchScript
    :param model_dir: 训练模型目录，包含bin文件、config.json
    :param export_dir: 导出模型目录
    :param device_id: 绑定硬件, cpu上运行：-1 | gpu上运行：0 or 1 or 2...
    :param example_inputs: 输入样例
    :param module: 神经网络
    :param kwargs: 网络自定义参数
    :return:
    读取权重示例
    model, tokenizer, state_dict = load_script_model(Path(r"F:\torch\script"))
    fc = nn.Linear(768, 1, bias=True)
    fc.weight.data = torch_to_script.get_state_dict_v(state_dict, "fc.weight")
    """
    bin_file = None
    if not export_dir.exists():
        export_dir.mkdir(parents=True)
    if module is None:
        config = AutoConfig.from_pretrained(pretrained_model_name_or_path=model_dir, **kwargs)
        model = AutoModel.from_pretrained(model_dir, config=config)
    else:
        model = module
    is_use_gpu = False if device_id == -1 else True
    if is_use_gpu:
        model = model.to("cuda:{0}".format(device_id))
    else:
        model = model.to("cpu")
    # 权重文件，这个是给预测的后处理模块初始化权重文件做准备
    for file in model_dir.glob("*.bin"):
        bin_file = file
        if not export_dir.joinpath(bin_file.name).exists():
            shutil.copy(bin_file, export_dir)
            break
    if module is not None:
        assert bin_file, '.bin file is not exists, please check {0}.'.format(model_dir)
        state_dict = torch.load(bin_file, map_location=get_device(device_id))
        model.load_state_dict(state_dict)
    model.eval()
    traced_model = torch.jit.trace(model, example_inputs=example_inputs, strict=False)
    if is_use_gpu:
        torch.jit.save(traced_model, export_dir.joinpath("traced_model_gpu.pt"))
    else:
        torch.jit.save(traced_model, export_dir.joinpath("traced_model_cpu.pt"))
    # tokenizer文件，这个是给预测的input data做准备
    try:
        tokenizer = AutoTokenizer.from_pretrained(model_dir)
        tokenizer.save_pretrained(export_dir)
    except Exception as e:
        pass
    print("done.")


def example_inputs_demo(device_id: int = -1, input_size=10, batch_size=128):
    """
    输入示例
    :param device_id: cpu上运行：-1 | gpu上运行：0 or 1 or 2...
    :param input_size:
    :param batch_size:
    :return:
    """
    check_device_id(device_id)
    ids = torch.LongTensor(input_size, batch_size).zero_()
    seq_len = torch.LongTensor(input_size, batch_size).zero_()
    mask = torch.LongTensor(input_size, batch_size).zero_()
    if device_id == -1:
        return [ids, seq_len, mask]
    else:
        check_cuda()
        return [ids.cuda(device_id), seq_len.cuda(device_id), mask.cuda(device_id)]


def get_state_dict_v(state_dict: dict, state_name: str):
    """
    根据名称获取权重值
    :param state_dict: 权重字典
    :param state_name: 名称
    :return:
    """
    for name in state_dict:
        if name == state_name:
            return state_dict.get(name)
    return None


def convert_bert_demo():
    """
    转换示例：以bert转torchScript为例
    :return:
    """
    """
    通用导出示例
    """
    # convert_script_model(module=torch.nn.Module(),
    #                      model_dir=Path(r"F:\torch\train_model"),
    #                      export_dir=Path(r"F:\torch\script"),
    #                      device="cpu", example_inputs=(example_inputs_demo(), ))
    """
    自定义导出示例（以bert导出为例）
    example_inputs       输入样例
    output_hidden_states 输出隐藏层
    output_attentions    输出意力层
    """
    args = example_inputs_demo(device_id=-1)
    args = args[0], args[1], args[2],
    convert_script_model(model_dir=Path(r"F:\torch\test"),
                         export_dir=Path(r"F:\torch\test"),
                         device="cpu", example_inputs=args,
                         torchscript=True, use_cache=False, output_hidden_states=True, output_attentions=True)


if __name__ == "__main__":
    """
    script转换示例
    """
    convert_bert_demo()
    """
    script加载示例
    """
    # model, tokenizer, state_dict = load_script_model(Path(r"F:\torch\script"))
    # fc_weight = get_state_dict_v(state_dict, "fc.weight")
    # print(fc_weight)
    pass



# -*- coding:utf-8 -*-
# Author:  zhousf
# Description:  list交集、并集、差集运算
import random
from math import ceil
from collections import Counter

from zhousflib.util import string_util


def counter(data_list: list, arg=None):
    """
    统计列表中某个元素的个数
    :param data_list: 列表
    :param arg: 某个元素，空时则返回counter
    :return:
    """
    res = Counter(data_list)
    if arg:
        return res.get(arg, 0)
    else:
        return res


def get_max_counter(data_list: list) -> int:
    """
    返回列表中重复项最多的个数
    :param data_list:  ['2', '2', '131.8', '131.8', '131.8', '131.8', '131.8', '1']
    :return: 5
    """
    if len(data_list) == 0:
        return 0
    res = Counter(data_list)
    count_list = [res.get(i) for i in res]
    count_list.sort(reverse=True)
    return count_list[0]


def get_min_counter(data_list: list) -> int:
    """
    返回列表中重复项最少的个数
    :param data_list:  ['2', '2', '131.8', '131.8', '131.8', '131.8', '131.8', '1']
    :return: 1
    """
    if len(data_list) == 0:
        return 0
    res = Counter(data_list)
    count_list = [res.get(i) for i in res]
    count_list.sort(reverse=False)
    return count_list[0]


def random_choices(data_list: list, choose_k=3) -> list:
    """
    从列表中随机抽取choose_k个数（会有重复值）
    :param data_list:
    :param choose_k:
    :return:
    """
    return random.choices(data_list, k=choose_k)


def none_filter(data: list) -> list:
    """
    去掉list中的None值
    :param data:
    :return:
    """
    if isinstance(data, list):
        res = []
        for item in data:
            if isinstance(item, list):
                res.append(list(filter(None, item)))
            else:
                res = list(filter(None, data))
                break
        return res
    return data


def intersection(a, b):
    """
    交集
    :param a: [1, 2, 3, 4, 5]
    :param b: [2, 3, 9]
    :return: [2, 3]
    """
    if len(a) == 0:
        return b
    if len(b) == 0:
        return a
    return list(set(a).intersection(set(b)))


def union(a, b):
    """
    并集
    :param a: [1, 2, 3, 4, 5]
    :param b: [2, 3, 9]
    :return: [1, 2, 3, 4, 5, 9]
    """
    if len(a) == 0:
        return b
    if len(b) == 0:
        return a
    return list(set(a).union(set(b)))


def difference(a, b):
    """
    差集
    :param a: [1, 2, 3, 4, 5]
    :param b: [2, 3, 9]
    :return: [1, 4, 5]
    """
    if len(a) == 0:
        return b
    if len(b) == 0:
        return a
    return list(set(a).difference(set(b)))


def sort(data: list, index: int = 0, reverse=False):
    """
    排序
    :param data:
    :param index: 排序参考的索引
    :param reverse: 是否倒序
    :return:
    """
    return sorted(data, key=lambda v: v[index], reverse=reverse)


def get_min_number(what: list):
    """
    获取list中最小值
    ["685.2", "10", "ab"]  -> 10
    :param what:
    :return:
    """
    min_value = ""
    number = [item for item in what if string_util.is_number(item)]
    for item in number:
        if string_util.is_less_than_number(item, min_value) or not min_value:
            min_value = item
    return min_value


def get_max_number(what: list):
    """
    获取list中最大值
    ["685.2", "10", "ab"]  -> 685.2
    :param what:
    :return:
    """
    max_value = ""
    number = [item for item in what if string_util.is_number(item)]
    for item in number:
        if string_util.is_greater_than_number(item, max_value) or not max_value:
            max_value = item
    return max_value


def delete_same_value(data_list: list, same_value_limit=1):
    """
    删除重复的值
    :param data_list:
    :param same_value_limit: 限制重复值的个数
    :return:
    ['113.8', '113.8', '131.8', '131.8', '131.8', '131.8', '131.8', "1"]
    处理成：
    ['113.8', '131.8', '1']
    """
    if len(data_list) == 0:
        return data_list
    counter_ = counter(data_list)
    repeat_num = [num for num in counter_.values() if num > 1]
    if len(repeat_num) > 1:
        for key in counter_.keys():
            num = counter_.get(key)
            if num > same_value_limit:
                counter_[key] = same_value_limit
    data_list.clear()
    for key in counter_.keys():
        for i in range(counter_.get(key)):
            data_list.append(key)
    return data_list


def chunk_list(data: list, chunk_size=5) -> list:
    """
    将数组按照每chunk_size个一组
    """
    if chunk_size <= 0:
        return data
    return [data[i * chunk_size:(i+1)*chunk_size] for i in range(0, ceil(len(data) / chunk_size))]



# -*- coding:utf-8 -*-
# Author:  zhousf
# Description:
import imghdr
import base64
import hashlib
from pathlib import Path

import cv2
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image, ImageDraw, ImageFont

from zhousflib.font import Font_SimSun
from zhousflib.util import pil_util


def get_file_base64(file_path: Path, contain_file_name=False, split_char=","):
    """
    图片转base64
    :param file_path: 图片路径
    :param contain_file_name: 是否包含文件名称
    :param split_char: 分隔符
    :return: 'a.jpg,iVBORw0KGgoAAAANSUhEUgAABNcAAANtCAYAAACzHZ25AAA.....'
    """
    with file_path.open('rb') as infile:
        s = infile.read()
    base64_str = base64.b64encode(s).decode("utf-8")
    if contain_file_name:
        base64_str = file_path.name + split_char + base64_str
    return base64_str


def md5(file_path: Path):
    with file_path.open('rb') as f:
        return hashlib.md5(f.read()).hexdigest()


def rename_image_with_md5(src_dir: Path, dst_dir: Path):
    if not dst_dir.exists():
        dst_dir.mkdir()
    count = 0
    repeat = 0
    for file in src_dir.rglob("*.*"):
        if not imghdr.what(str(file)):
            continue
        print(file.name)
        count += 1
        new_name = md5(file)
        new_name += file.suffix
        print(new_name)
        if dst_dir.joinpath(new_name).exists():
            repeat += 1
            continue
        if not dst_dir.joinpath(file.parent.name).exists():
            dst_dir.joinpath(file.parent.name).mkdir(parents=True)
        if not dst_dir.joinpath(file.parent.name).joinpath(new_name).exists():
            file.rename(dst_dir.joinpath(file.parent.name).joinpath(new_name))
    print("count=", count)
    print("repeat=", repeat)


def get_image_binary(image_path: Path, **kwargs):
    """
    获取二值化图片
    :param image_path:
    :param kwargs: 二值化参数
    :return:
    """
    assert image_path.exists(), "image file is not exists: {0}".format(image_path)
    img = cv2.imread(str(image_path), cv2.IMREAD_GRAYSCALE)
    assert img is not None, "image is None"
    if len(kwargs) > 0:
        binary = cv2.adaptiveThreshold(src=img, **kwargs)
    else:
        binary = cv2.adaptiveThreshold(src=img,
                                       maxValue=255,  # 大于阈值后设定的值
                                       adaptiveMethod=cv2.ADAPTIVE_THRESH_MEAN_C,  # 自适应方法
                                       thresholdType=cv2.THRESH_BINARY,  # 同全局阈值法中的参数一样
                                       blockSize=11,  # 方阵（区域）大小
                                       C=15)  # 常数项
    return binary


def filter_pixels_at_edges(image_path: Path, save_image_path: Path = None, distance_at_edges=20, show=False, **kwargs):
    """
    删除图片边缘的像素块
    :param image_path: 待处理图片路径
    :param save_image_path: 保存删除后的图片路径
    :param distance_at_edges: 像素块离图片边缘的距离，小于该值则判定该像素块是边缘上的
    :param show: 显示边缘上的像素块
    :param kwargs: 二值化参数
    :return:
    filter_pixels_at_edges(image_path=Path(r'0.jpg'), save_image_path=Path(r'0-1.jpg'), show=False)
    """
    binary = get_image_binary(image_path=image_path, **kwargs)
    img_height, img_width = binary.shape
    # 掩码标识转换
    arr_mask = np.where(binary == 0, 255, 0)
    # 掩码类型转换
    arr_mask = arr_mask.astype(dtype=np.uint8)
    nums, labels, stats, centroids = cv2.connectedComponentsWithStats(arr_mask, connectivity=4)
    background = 0
    for row in range(stats.shape[0]):
        if stats[row, :][0] == 0 and stats[row, :][1] == 0:
            background = row
    # 删除背景后的连通域列表
    stats_no_bg = np.delete(stats, background, axis=0).tolist()
    centroids_no_bg = np.delete(centroids, background, axis=0).tolist()
    domain_list = []
    boxes = []
    for i in range(0, len(stats_no_bg)):
        box_x, box_y, box_width, box_height, area = stats_no_bg[i]
        center_point_x, center_point_y = centroids_no_bg[i]
        domain_list.append((center_point_x, center_point_y, box_x, box_y, box_width, box_height, area))
        x_min = box_x
        y_min = box_y
        x_max = x_min + box_width
        y_max = y_min + box_height
        boxes.append((i, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max))
    boxes = sorted(boxes, key=lambda v: v[1], reverse=True)
    for i in range(len(boxes) - 1, -1, -1):
        index, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max = boxes[i]
        # 过滤在图片边缘的文字
        if abs(img_width - center_point_x) < distance_at_edges or abs(img_height - center_point_y) < distance_at_edges \
                or center_point_y < distance_at_edges or center_point_x < distance_at_edges:
            pass
        else:
            boxes.pop(i)
    img_vis = pil_util.draw_rectangle(bbox=boxes, image_file=image_path, fill_transparent=100, show=False,
                                      font=ImageFont.truetype(font=str(Font_SimSun), size=20, encoding="utf-8"))
    """
    删除像素
    """
    image = Image.open(image_path)
    if image.mode != "RGB":
        image = image.convert('RGB')
    draw = ImageDraw.Draw(image)
    for i in range(len(boxes) - 1, -1, -1):
        index, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max = boxes[i]
        # 将box以白色填充
        draw.rectangle((x_min, y_min, x_max, y_max), fill=(255, 255, 255))
    if show:
        plt.figure(figsize=(16, 9))
        plt.subplot(131)
        plt.imshow(Image.open(image_path))
        plt.title("input")
        plt.subplot(132)
        plt.imshow(img_vis)
        plt.title("pixels_at_edges")
        plt.subplot(133)
        plt.imshow(image)
        plt.title("output")
        plt.show()
        plt.close("all")
    if save_image_path is not None:
        image.save(save_image_path, quality=100)
    return image


def filter_pixels_by_area(image_path: Path, save_image_path: Path = None, block_area=100, show=False, **kwargs):
    """
    根据像素块面积删除图片的像素块
    :param image_path: 待处理图片路径
    :param save_image_path: 保存删除后的图片路径
    :param block_area: 删除像素块的面积阈值，小于该值则删除该像素块
    :param show: 显示边缘上的像素块
    :param kwargs: 二值化参数
    :return:
    """
    binary = get_image_binary(image_path=image_path, **kwargs)
    # 掩码标识转换
    arr_mask = np.where(binary == 0, 255, 0)
    # 掩码类型转换
    arr_mask = arr_mask.astype(dtype=np.uint8)
    nums, labels, stats, centroids = cv2.connectedComponentsWithStats(arr_mask, connectivity=4)
    background = 0
    for row in range(stats.shape[0]):
        if stats[row, :][0] == 0 and stats[row, :][1] == 0:
            background = row
    # 删除背景后的连通域列表
    stats_no_bg = np.delete(stats, background, axis=0).tolist()
    centroids_no_bg = np.delete(centroids, background, axis=0).tolist()
    domain_list = []
    boxes = []
    for i in range(0, len(stats_no_bg)):
        box_x, box_y, box_width, box_height, area = stats_no_bg[i]
        center_point_x, center_point_y = centroids_no_bg[i]
        domain_list.append((center_point_x, center_point_y, box_x, box_y, box_width, box_height, area))
        x_min = box_x
        y_min = box_y
        x_max = x_min + box_width
        y_max = y_min + box_height
        boxes.append((i, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max))
    boxes = sorted(boxes, key=lambda v: v[1], reverse=True)
    for i in range(len(boxes) - 1, -1, -1):
        index, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max = boxes[i]
        if area > block_area:
            boxes.pop(i)
    img_vis = pil_util.draw_rectangle(bbox=boxes, image_file=image_path, fill_transparent=100, show=False,
                                      font=ImageFont.truetype(font=str(Font_SimSun), size=20, encoding="utf-8"))
    """
    删除像素
    """
    image = Image.open(image_path)
    if image.mode != "RGB":
        image = image.convert('RGB')
    draw = ImageDraw.Draw(image)
    for i in range(len(boxes) - 1, -1, -1):
        index, area, center_point_x, center_point_y, x_min, y_min, x_max, y_max = boxes[i]
        # 将box以白色填充
        draw.rectangle((x_min, y_min, x_max, y_max), fill=(255, 255, 255))
    if show:
        plt.figure(figsize=(16, 9))
        plt.subplot(131)
        plt.imshow(Image.open(image_path))
        plt.title("input")
        plt.subplot(132)
        plt.imshow(img_vis)
        plt.title("pixels_at_edges")
        plt.subplot(133)
        plt.imshow(image)
        plt.title("output")
        plt.show()
        plt.close("all")
    if save_image_path is not None:
        image.save(save_image_path, quality=100)
    return image


if __name__ == "__main__":
    filter_pixels_at_edges(image_path=Path(r'C:\Users\zhousf-a\Desktop\0.jpg'), show=True)
    # filter_pixels_by_area(image_path=Path(r'C:\Users\zhousf-a\Desktop\0.jpg'), show=True)
    pass



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/12/14 
# @Function:

"""
官方文档：https://docs.nvidia.com/deeplearning/triton-inference-server/user-guide/docs/user_guide/performance_tuning.html
官方git：https://github.com/triton-inference-server/server/blob/main/docs/README.md#user-guide
# 创建模型仓库目录：
triton/models/模型名称/版本号/模型
# 在模型名称目录下创建模型配置文件：
参考：https://github.com/triton-inference-server/server/blob/main/docs/user_guide/model_configuration.md
config.pbtxt
# 创建一个模型仓库容器并启动一个模型推理服务（gpus=all）
在triton目录下运行：
docker run -it --rm --gpus '"device=1,2,3"' --network=host -v $PWD:/mnt --name zhousf-triton-server nvcr.io/nvidia/tritonserver:23.11-py3 tritonserver --model-repository=/mnt/models --http-port=5005 --grpc-port=5006 --metrics-port=5007 --log-info=true --log-error=true
docker run -it -d --rm --gpus=all --network=host -v $PWD:/mnt --name zhousf-triton-server nvcr.io/nvidia/tritonserver:23.11-py3 tritonserver --model-repository=/mnt/models --http-port=5005 --grpc-port=5006 --metrics-port=5007 --log-info=true --log-error=true
# 显示如下表示成功：
+---------------+---------+--------+
| Model         | Version | Status |
+---------------+---------+--------+
| cosnet_onnx   | 1       | READY  |
+---------------+---------+--------+
# 测试模型服务是否启动成功
curl -v localhost:5005/v2/health/ready

# 吞吐量测试
docker pull nvcr.io/nvidia/tritonserver:23.08-py3-sdk
docker run --gpus all --rm -it --net=host --name zhousf-perf_analyzer nvcr.io/nvidia/tritonserver:23.08-py3-sdk
perf_analyzer -m cosnet_onnx --shape attention_mask:10,128 --shape input_ids:10,128 --shape token_type_ids:10,128  -i http --concurrency-range 1:50:10 -u 0.0.0.0:5005
perf_analyzer -m cosnet_onnx --shape attention_mask:10,128 --shape input_ids:10,128 --shape token_type_ids:10,128  -i grpc --concurrency-range 1:50:10 -u 0.0.0.0:5006

"""


"""
name: "cosnet_onnx"
backend: "onnxruntime"
max_batch_size: 0
input: [
  {
    name: "input_ids",
    data_type: TYPE_INT64,
    dims: [ -1, 128]
  },
  {
    name: "attention_mask",
    data_type: TYPE_INT64,
    dims: [ -1, 128]
  },
  {
    name: "token_type_ids",
    data_type: TYPE_INT64,
    dims: [ -1, 128]
  }
]
output: [
  {
    name: "output",
    data_type: TYPE_FP16,
    dims: [ -1 ]
  }
dynamic_batching {
    max_queue_delay_microseconds: 10
}
instance_group [
  {
    count: 4
    kind: KIND_GPU
  }
]
"""

"""
name: "cosnet_tensorrt"
platform: "tensorrt_plan"
max_batch_size: 0
input: [
  {
    name: "input_ids"
    data_type: TYPE_INT32
    dims: [ 50, 128]
  },
  {
    name: "attention_mask"
    data_type: TYPE_INT32
    dims: [ 50, 128]
  },
  {
    name: "token_type_ids"
    data_type: TYPE_INT32
    dims: [ 50, 128]
  }
]
output: [
  {
    name: "output"
    data_type: TYPE_FP16
    dims: [ -1 ]
  }
]
instance_group [
  {
    count: 2
    kind: KIND_GPU
  }
]
"""




# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: label-studio
import json
from prettytable import PrettyTable


def data_statistics(result_jsons: list):
    """
    数据分布统计
    :param result_jsons: 数据集目录
    :return:
    """
    image_total = 0
    label_list = {}
    for label_json in result_jsons:
        with label_json.open("r", encoding="utf-8") as f:
            result_json = json.loads(f.read())
        # 标签类别
        label_list = result_json["categories"]
        break
    if len(label_list) == 0:
        print("标签类别无效")
        return
    label_names = {k.get("id"): k.get("name") for k in label_list}
    statistics_total = {k.get("id"): 0 for k in label_list}
    for label_json in result_jsons:
        with label_json.open("r", encoding="utf-8") as f:
            result_json = json.loads(f.read())
        # 图片
        image_total += len(result_json["images"])
        # 标注
        for label in result_json["annotations"]:
            label_id = label.get("category_id")
            statistics_total[label_id] += 1
    statistics_total = {label_names.get(k): statistics_total.get(k) for k in statistics_total}
    print(statistics_total)
    table = PrettyTable(field_names=["label", "count"])
    for key in statistics_total:
        table.add_row([key, statistics_total.get(key)])
    print(table)


# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Date    : 2023/11/1 
# @Function:
import shutil
from pathlib import Path


def delete_file_by_conditions(src_dir: Path,
                              include_file_name: list = None,
                              include_file_stem_startswith: list = None,
                              include_file_stem_endswith: list = None,
                              include_dir_name: list = None,
                              include_dir_stem_startswith: list = None,
                              include_dir_stem_endswith: list = None):
    """
    删除文件/目录：多条件
    :param src_dir: 目录
    :param include_file_name: 文件名称为["log.txt", "error.txt"]
    :param include_file_stem_startswith: 文件名以["_vis", "_loc", "_ocr"]开始
    :param include_file_stem_endswith:  文件名以...结尾
    :param include_dir_name:  目录名称为["__pycache__", ".DS_Store", "cls"]
    :param include_dir_stem_startswith:
    :param include_dir_stem_endswith:
    :return:
    """
    for file in src_dir.rglob("*"):
        print(file)
        if file.is_dir():
            if include_dir_name:
                for dir_name in include_dir_name:
                    if file.name == dir_name:
                        shutil.rmtree(file)
                        continue
            if include_dir_stem_startswith:
                for dir_name in include_dir_stem_startswith:
                    if file.stem.startswith(dir_name):
                        shutil.rmtree(file)
                        continue
            if include_dir_stem_endswith:
                for dir_name in include_dir_stem_endswith:
                    if file.stem.endswith(dir_name):
                        shutil.rmtree(file)
                        continue
        if file.is_file():
            if include_file_name:
                for file_name in include_file_name:
                    if file.name == file_name:
                        file.unlink()
                        continue
            if include_file_stem_startswith:
                for file_name in include_file_stem_startswith:
                    if file.stem.startswith(file_name):
                        file.unlink()
                        continue
            if include_file_stem_endswith:
                for file_name in include_file_stem_endswith:
                    if file.stem.endswith(file_name):
                        file.unlink()
                        continue


def delete_file_by_suffix(src_dirs: list, delete_file_suffix: list = None):
    """
    删除文件: 通过文件扩展名
    :param src_dirs: 目录: [Path("")]
    :param delete_file_suffix: 文件扩展名，[".json"]
    :return:
    """
    if delete_file_suffix is None:
        delete_file_suffix = [".json"]
    delete = 0
    for src_dir in src_dirs:
        for suffix in delete_file_suffix:
            for file in src_dir.rglob("*{0}".format(suffix)):
                if not file.is_file():
                    continue
                file.unlink()
                delete += 1
                print(delete, file)
    print("共删除{0}项".format(delete))
